#!/usr/bin/env python
# -*- coding: utf-8 -*-
# encoding: utf-8
'''
messages.py

Processes messages utility library

Created by SWS on 2012-09-11.
Copyright (c) 2012 StoneWorkSolutions. All rights reserved.
'''
from django.conf import settings
import json
import logging
import re
import redis
import sys,traceback
import django.db.models
# import ooint.settings

logger = logging.getLogger('sws-tags')

def processErrorsForm(form):
	
	message = ''
	messages = []

	for field in form:

		if field.errors:
			message = re.sub('</(.*)>', '', unicode(str(field.errors), 'utf-8'))
			message = re.sub('<(.*)>', '', message)
			message = field.html_name + ': ' + message
			messages.append(message)

	return messages


from django_tools.middlewares import ThreadLocal
# ThreadLocal.get_current_user()
# si necesitaras toda la request..
# ThreadLocal.get_current_request()
	
def processMessages(message, typeMessage, timeOut=10000):

	msg = {}

	def getTraceBackInfo():
		info = inspect.stack()[2][3]
		return info 
		# print 'f_back',inspect.frame().f_back
		# mod = inspect.getmodule(frm[0])
		# print '[%s] %s' % (mod.__name__, msg)
	
	called_from_view = getTraceBackInfo()

	if typeMessage == 'error':
		try:
			logger.error(' view: '+called_from_view+', user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
		error = (message, timeOut)
		msg['error'] = error

	elif typeMessage == 'warning':
		try:
			logger.warning(' view: '+called_from_view+', user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
		warning = (message, timeOut)
		msg['warning'] = warning

	elif typeMessage == 'success':
		try:
			logger.info(' view: '+called_from_view+', user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
		success = (message, timeOut)
		msg['success'] = success

	elif typeMessage == 'info':
		try:
			logger.info(' view: '+called_from_view+', user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
		info = (message, timeOut)
		msg['info'] = info

	return msg

# Send message to the specific channel
# Parametres: 
# 	name channel (session_id or settings.CHANNEL_GLOBAL_MESSAGES)
# 	text message
# 	level (info, warning,error or debug)
# 	time visible
import inspect

def sendMessages(channel=settings.CHANNEL_GLOBAL_MESSAGES,message='No messages',level='info',time=5000):
	cr = redis.Redis(settings.REDIS_IP,db=settings.REDIS_DB)
	message={"message":message,"level":level,"time":time}
	cr.publish(channel,json.dumps(['user_message',message]))

	if level == 'error':
		try:
			logger.error(' user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
	elif level == 'warning':
		try:
			logger.warning(' user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
	elif level == 'success':
		try:
			logger.info(' user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass
	elif level == 'info':
		try:
			logger.info(' user: '+str(ThreadLocal.get_current_user())+', message: '+message)
		except:
			pass

	return True

def sendEvent(channel,event,data):
	cr = redis.Redis(settings.REDIS_IP,db=settings.REDIS_DB)
	cr.publish(channel,json.dumps([event,data]))

	try:
		logger.debug('New event\n-User: {0}\n-Event: {1}\n-Data: {2}'.format(ThreadLocal.get_current_user(),event,data))
	except:
		pass

	return True

def sendProgressBarEvent(channel, view_name, value, message='', error=False):
	sendEvent(channel,'progressbar',{'view_name':view_name,'value':value,'message':message, 'error': error})	

def sendBubbleNotificationEvent(channel, view_name, value, mode='add'):
	sendEvent(channel,'bubblenotification',{'view_name':view_name,'value':value,'mode':mode})	

def sendBubbleNotificationBlinkEvent(channel, view_name):
	sendEvent(channel,'bubblenotificationblink',{'view_name':view_name})	


def sendDashboardEvent(channel, data):
	sendEvent(channel,'dashboard',{'data':data})	



def swslog(typeMessage,message,e):

	msg = {}

	def getTraceBackInfo():
		info = inspect.stack()[2][3]
		return info 
	
	called_from_view = getTraceBackInfo()

	if typeMessage == 'error':
		try:
			logger.error(' -- '+called_from_view+', USER: '+str(ThreadLocal.get_current_user())+', MESSAGE ERROR: '+message+ ' EXCEPTION: {0}'.format(traceback.format_exc()))		
			# logger.error('*FUNCTION: '+called_from_view+', USER: '+str(ThreadLocal.get_current_user())+', MESSAGE ERROR: '+message)
			# logger.error('-----EXCEPTION: {0}'.format(traceback.format_exc()))

		except:
			pass

	elif typeMessage == 'warning':
		try:
			logger.warning(' -- '+called_from_view+', USER: '+str(ThreadLocal.get_current_user())+', MESSAGE WARNING: '+message+ ' EXCEPTION: {0}'.format(traceback.format_exc()))
		except:
			pass

	elif typeMessage == 'success':
		try:
			logger.info(' -- '+called_from_view+', USER: '+str(ThreadLocal.get_current_user())+', MESSAGE SUCCES: '+message+ 'EXCEPTION: {0}'.format(traceback.format_exc()))
		except:
			pass

	elif typeMessage == 'info':
		try:
			logger.info(' -- '+called_from_view+', USER: '+str(ThreadLocal.get_current_user())+', MESSAGE INFO: '+message+ 'EXCEPTION: {0}'.format(traceback.format_exc()))
		except:
			pass




#!/usr/bin/env python
# encoding: utf-8

from openpyxl import Workbook
from openpyxl.writer.excel import save_virtual_workbook
from sws_tags.sws_utils import json_encode

import csv
from django.http import HttpResponse
from django.template import loader, Context
from decimal import Decimal
from datetime import date, datetime, timedelta
import time
import ujson
# import datetime

# FOR EXCEL GENERATING
import xlwt
ezxf = xlwt.easyxf

from sws_tags.sws_utils.common_utils import *
from sws_tags.sws_utils.messages import *
from sws_tags.sws_utils.cube import *

import traceback


def ExportFile(param_export_file):


	queryset = param_export_file['queryset']
	headers = param_export_file['headers']
	col_name = param_export_file['col_name']
	filename = param_export_file['filename']
	logger = param_export_file['logger']


	wb = Workbook()
	ws = wb.get_active_sheet()
	ws.title = filename
	row = 0
	col = 0
	for h in headers:
		cell = ws.cell(row = row, column = col)
		cell.value = h
		col += 1
	row = 1
	col = 0
	for trr in queryset:
		for c in col_name:
			cell = ws.cell(row = row, column = col)
			col += 1
			cell.value = trr[c]
		row += 1
		col = 0


	# print'aaaaaaaaaaaaaaaaaaaaaaaaa',wb

	wb.save("media/excel/{0}".format(filename))
	logger.debug('Excel file saved in media/excel/{0}'.format(filename))
	data =  {
			'msg': '{0} excel file generated, please press download button.'.format(filename),
			'msg_type': 'msg_success',
			'file_url':"excel/{0}".format(filename),
			}
	json_data = json_encode(data)
	return wb




def get_csv_query(request, queryset, filename, col_order = None):

	kind_to_xf_map = {
		'date': ezxf(num_format_str='yyyy-mm-dd HH:MM'),
		'int': ezxf(num_format_str='#,##0'),
		'money': ezxf('font: italic on; pattern: pattern solid, fore-colour grey25',
			num_format_str='€#,##0.00'),
		'price': ezxf(num_format_str='#0.000'),
		'text': ezxf(),
		'boolean': ezxf(),
	}

	try:
		# csv_file = open(filename,"w")
		csv_file = HttpResponse()

		csv_writer = csv.writer(csv_file, dialect='excel', quoting=csv.QUOTE_MINIMAL, delimiter='|')

		if type(queryset)!=list:
			q = queryset[0]

			
			fields = []
			for f in queryset._fields:
				fields.append(q[f])
			for f in queryset.aggregate_names:
				fields.append(q[f])

			fields_name = []
			for f in queryset._fields:
				fields_name.append(unicode(getFieldVerboseName(queryset,f)))
			for f in queryset.aggregate_names:
				fields_name.append(f)


			data_xfs_types_used = [k for k in get_field_types(fields)]
			data_xfs = [kind_to_xf_map[k] for k in get_field_types(fields)]

			data = []


			for i, q in enumerate(queryset):
				aux = []
				i=0
				for f in queryset._fields:

					if data_xfs_types_used[i] == 'date':
						date_normalize=request['django_timezone'].normalize(q[f]).strftime('%Y-%m-%d %H:%M:%S')
						aux.append(str(date_normalize))
					else:
						aux.append(q[f])
					i+=1
				for f in queryset.aggregate_names:
					aux.append(q[f])
				data.append(aux)

		else:
			fields_name = []
			if col_order:
				fields_name=col_order
			else:
				for k,v in queryset[0].items():
					fields_name.append(unicode(k))

			data = []

			for q in queryset:
				v_data=[]
				for k in fields_name:
					# print 'excel-->',q[k],'--',k
					v_data.append(q[k])
				data.append(v_data)

		csv_writer.writerow(fields_name)

		for row in data:
			csv_writer.writerow(row)							
								
		# csv_file.close()
		return csv_file

	except Exception, e:
		swslog('error','get_csv_query',e)
		return False


